@@ -117,6 +117,7 @@ typedef struct
   GPid  pid;
   char *client;
   guint child_watch;
+  gboolean watch_bus;
 } PidData;
 
 static void
@@ -218,7 +219,7 @@ child_setup_func (gpointer user_data)
   if (data->set_tty)
     {
       /* data->tty is our from fd which is closed at this point.
-       * so locate the destnation fd and use it for the ioctl.
+       * so locate the destination fd and use it for the ioctl.
        */
       for (i = 0; i < data->fd_map_len; i++)
         {
@@ -321,6 +322,7 @@ handle_spawn (PortalFlatpak         *object,
   g_auto(GStrv) sandbox_expose = NULL;
   g_auto(GStrv) sandbox_expose_ro = NULL;
   gboolean sandboxed;
+  gboolean devel;
 
   app_info = g_object_get_data (G_OBJECT (invocation), "app-info");
   g_assert (app_info != NULL);
@@ -329,9 +331,45 @@ handle_spawn (PortalFlatpak         *object,
                                   FLATPAK_METADATA_GROUP_APPLICATION,
                                   FLATPAK_METADATA_KEY_NAME, NULL);
   g_assert (app_id != NULL);
+
+  g_debug ("spawn() called from app: '%s'", app_id);
+  if (*app_id == 0)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                             G_DBUS_ERROR_INVALID_ARGS,
+                                             "org.freedesktop.portal.Flatpak.Spawn only works in a flatpak");
+      return TRUE;
+    }
+
+
+  if (*arg_cwd_path == 0)
+    arg_cwd_path = NULL;
+
+  if (arg_argv == NULL || *arg_argv == NULL)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
+                                             G_DBUS_ERROR_INVALID_ARGS,
+                                             "No command given");
+      return TRUE;
+    }
+
+  if ((arg_flags & ~FLATPAK_SPAWN_FLAGS_ALL) != 0)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
+                                             "Unsupported flags enabled: 0x%x", arg_flags & ~FLATPAK_SPAWN_FLAGS_ALL);
+      return TRUE;
+    }
+
   runtime_ref = g_key_file_get_string (app_info,
                                        FLATPAK_METADATA_GROUP_APPLICATION,
                                        FLATPAK_METADATA_KEY_RUNTIME, NULL);
+  if (runtime_ref == NULL)
+    {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
+                                             "No runtime found");
+      return TRUE;
+    }
+
   runtime_parts = g_strsplit (runtime_ref, "/", -1);
 
   branch = g_key_file_get_string (app_info,
@@ -355,26 +393,8 @@ handle_spawn (PortalFlatpak         *object,
   shares = g_key_file_get_string_list (app_info, FLATPAK_METADATA_GROUP_CONTEXT,
                                        FLATPAK_METADATA_KEY_SHARED, NULL, NULL);
 
-  g_debug ("spawn() called from app: %s", app_id);
-
-  if (*app_id == 0)
-    {
-      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
-                                             G_DBUS_ERROR_INVALID_ARGS,
-                                             "org.freedesktop.portal.Flatpak.Spawn only works in a flatpak");
-      return TRUE;
-    }
-
-  if (*arg_cwd_path == 0)
-    arg_cwd_path = NULL;
-
-  if (arg_argv == NULL || *arg_argv == NULL)
-    {
-      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR,
-                                             G_DBUS_ERROR_INVALID_ARGS,
-                                             "No command given");
-      return TRUE;
-    }
+  devel = g_key_file_get_boolean (app_info, FLATPAK_METADATA_GROUP_INSTANCE,
+                                  FLATPAK_METADATA_KEY_DEVEL, NULL);
 
   g_variant_lookup (arg_options, "sandbox-expose", "^as", &sandbox_expose);
   g_variant_lookup (arg_options, "sandbox-expose-ro", "^as", &sandbox_expose_ro);
@@ -508,6 +528,9 @@ handle_spawn (PortalFlatpak         *object,
         g_ptr_array_add (flatpak_argv, g_strdup (extra_args[i]));
     }
 
+  if (devel)
+    g_ptr_array_add (flatpak_argv, g_strdup ("--devel"));
+
   /* Inherit launcher network access from launcher, unless
      NO_NETWORK set. */
   if (shares != NULL && g_strv_contains ((const char * const *) shares, "network") &&
@@ -543,6 +566,8 @@ handle_spawn (PortalFlatpak         *object,
         g_ptr_array_add (flatpak_argv, g_strdup_printf ("--runtime-commit=%s", runtime_commit));
     }
 
+  if (arg_cwd_path != NULL)
+    g_ptr_array_add (flatpak_argv, g_strdup_printf ("--cwd=%s", arg_cwd_path));
 
   if (arg_argv[0][0] != 0)
     g_ptr_array_add (flatpak_argv, g_strdup_printf ("--command=%s", arg_argv[0]));
@@ -567,7 +592,7 @@ handle_spawn (PortalFlatpak         *object,
       g_debug ("Starting: %s\n", cmd->str);
     }
 
-  if (!g_spawn_async_with_pipes (arg_cwd_path,
+  if (!g_spawn_async_with_pipes (NULL,
                                  (char **) flatpak_argv->pdata,
                                  env,
                                  G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
@@ -592,6 +617,7 @@ handle_spawn (PortalFlatpak         *object,
   pid_data = g_new0 (PidData, 1);
   pid_data->pid = pid;
   pid_data->client = g_strdup (g_dbus_method_invocation_get_sender (invocation));
+  pid_data->watch_bus = (arg_flags & FLATPAK_SPAWN_FLAGS_WATCH_BUS) != 0;
   pid_data->child_watch = g_child_watch_add_full (G_PRIORITY_DEFAULT,
                                                   pid,
                                                   child_watch_died,
@@ -675,6 +701,52 @@ authorize_method_handler (GDBusInterfaceSkeleton *interface,
   return TRUE;
 }
 
+static void
+name_owner_changed (GDBusConnection *connection,
+                    const gchar     *sender_name,
+                    const gchar     *object_path,
+                    const gchar     *interface_name,
+                    const gchar     *signal_name,
+                    GVariant        *parameters,
+                    gpointer         user_data)
+{
+  const char *name, *from, *to;
+
+  g_variant_get (parameters, "(&s&s&s)", &name, &from, &to);
+
+  if (name[0] == ':' &&
+      strcmp (name, from) == 0 &&
+      strcmp (to, "") == 0)
+    {
+      GHashTableIter iter;
+      PidData *pid_data = NULL;
+      gpointer value = NULL;
+      GList *list = NULL, *l;
+
+      g_hash_table_iter_init (&iter, client_pid_data_hash);
+      while (g_hash_table_iter_next (&iter, NULL, &value))
+        {
+          pid_data = value;
+
+          if (pid_data->watch_bus && g_str_equal (pid_data->client, name))
+            list = g_list_prepend (list, pid_data);
+        }
+
+      for (l = list; l; l = l->next)
+        {
+          pid_data = l->data;
+          g_debug ("%s dropped off the bus, killing %d", pid_data->client, pid_data->pid);
+          killpg (pid_data->pid, SIGINT);
+        }
+
+      g_list_free (list);
+    }
+}
+
+#define DBUS_NAME_DBUS "org.freedesktop.DBus"
+#define DBUS_INTERFACE_DBUS DBUS_NAME_DBUS
+#define DBUS_PATH_DBUS "/org/freedesktop/DBus"
+
 static void
 on_bus_acquired (GDBusConnection *connection,
                  const gchar     *name,
@@ -684,8 +756,20 @@ on_bus_acquired (GDBusConnection *connection,
 
   g_debug ("Bus acquired, creating skeleton");
 
+  g_dbus_connection_set_exit_on_close (connection, FALSE);
+
   portal = portal_flatpak_skeleton_new ();
 
+  g_dbus_connection_signal_subscribe (connection,
+                                      DBUS_NAME_DBUS,
+                                      DBUS_INTERFACE_DBUS,
+                                      "NameOwnerChanged",
+                                      DBUS_PATH_DBUS,
+                                      NULL,
+                                      G_DBUS_SIGNAL_FLAGS_NONE,
+                                      name_owner_changed,
+                                      NULL, NULL);
+
   g_object_set_data_full (G_OBJECT (portal), "track-alive", GINT_TO_POINTER (42), skeleton_died_cb);
 
   g_dbus_interface_skeleton_set_flags (G_DBUS_INTERFACE_SKELETON (portal),
