@@ -4,7 +4,7 @@ set -e
 
 DIR=`mktemp -d`
 
-REPONAME=$1
+REPO=$1
 shift
 APP_ID=$1
 shift
@@ -25,18 +25,20 @@ cat > ${DIR}/metadata <<EOF
 name=$APP_ID
 runtime=org.test.Platform/$ARCH/master
 sdk=org.test.Platform/$ARCH/master
-EOF
 
-if [ x${REQUIRED_VERSION-} != x ]; then
-cat >> ${DIR}/metadata <<EOF
-required-flatpak=$REQUIRED_VERSION
+[Extension org.test.Hello.Locale]
+directory=share/runtime/locale
+autodelete=true
+locale-subset=true
 EOF
-fi
 
 mkdir -p ${DIR}/files/bin
 cat > ${DIR}/files/bin/hello.sh <<EOF
 #!/bin/sh
 echo "Hello world, from a sandbox$EXTRA"
+if [ "$EXTRA" = "SPIN" ]; then
+  exec sh
+fi
 EOF
 chmod a+x ${DIR}/files/bin/hello.sh
 
@@ -72,6 +74,9 @@ gzip -c > ${DIR}/files/share/app-info/xmls/${APP_ID}.xml.gz <<EOF
       <category>Utility</category>
     </categories>
     <icon height="64" width="64" type="cached">64x64/org.gnome.gedit.png</icon>
+    <releases>
+      <release timestamp="1525132800" version="0.0.1"/>
+    </releases>
   </component>
 </components>
 EOF
@@ -83,7 +88,46 @@ else
     collection_args=
 fi
 
+mkdir -p ${DIR}/files/share/locale
+mkdir -p ${DIR}/files/share/runtime/locale/de
+ln -s -t ${DIR}/files/share/locale ../../share/runtime/locale/de/share/de
+mkdir -p ${DIR}/files/share/runtime/locale/fr
+ln -s -t ${DIR}/files/share/locale ../../share/runtime/locale/fr/share/fr
+
 flatpak build-finish --command=hello.sh ${DIR}
 mkdir -p repos
-flatpak build-export ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} repos/${REPONAME} ${DIR}
+flatpak build-export --disable-sandbox ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} ${REPO} ${DIR}
 rm -rf ${DIR}
+
+# build a locale extension
+
+DIR=`mktemp -d`
+
+# Init dir
+cat > ${DIR}/metadata <<EOF
+[Runtime]
+name=${APP_ID}.Locale
+
+[ExtensionOf]
+ref=app/$APP_ID/$ARCH/master
+EOF
+
+cat > de.po <<EOF
+msgid "Hello world"
+msgstr "Hallo Welt"
+EOF
+mkdir -p ${DIR}/files/de/share/de/LC_MESSAGES
+msgfmt --output-file ${DIR}/files/de/share/de/LC_MESSAGES/helloworld.mo de.po
+cat > fr.po <<EOF
+msgid "Hello world"
+msgstr "Bonjour le monde"
+EOF
+mkdir -p ${DIR}/files/fr/share/fr/LC_MESSAGES
+msgfmt --output-file ${DIR}/files/fr/share/fr/LC_MESSAGES/helloworld.mo fr.po
+
+flatpak build-finish ${DIR}
+mkdir -p repos
+flatpak build-export --runtime ${collection_args} ${GPGARGS-} ${EXPORT_ARGS-} ${REPO} ${DIR}
+rm -rf ${DIR}
+
+
