@@ -23,7 +23,7 @@ set -euo pipefail
 
 skip_without_bwrap
 
-echo "1..23"
+echo "1..27"
 
 #Regular repo
 setup_repo
@@ -130,6 +130,45 @@ echo "ok failed to install again from different remote"
 ${FLATPAK} ${U} install -y --reinstall test-repo org.test.Platform
 echo "ok re-install"
 
+${FLATPAK} ${U} uninstall -y org.test.Hello
+
+# Note: This typo is only auto-corrected without user interaction because we're using -y
+${FLATPAK} ${U} install -y test-repo org.test.Hllo >install-log
+assert_file_has_content install-log "org.test.Hello"
+
+${FLATPAK} ${U} list -d > list-log
+assert_file_has_content list-log "org.test.Hello"
+
+echo "ok typo correction works for install"
+
+${FLATPAK} ${U} uninstall -y org.test.Hello
+
+# Temporarily disable some remotes so that org.test.Hello only exists in one
+${FLATPAK} ${U} remote-modify --disable test-missing-gpg-repo
+${FLATPAK} ${U} remote-modify --disable test-wrong-gpg-repo
+${FLATPAK} ${U} remote-modify --disable test-gpg2-repo
+${FLATPAK} ${U} remote-modify --disable local-test-no-gpg-repo
+if [ x${USE_COLLECTIONS_IN_CLIENT-} != xyes ] ; then
+    ${FLATPAK} ${U} remote-modify --disable test-no-gpg-repo
+fi
+
+# Note: The missing remote is only auto-corrected without user interaction because we're using -y
+${FLATPAK} ${U} install -y org.test.Hello |& tee install-log
+assert_file_has_content install-log "org.test.Hello"
+
+${FLATPAK} ${U} list -d > list-log
+assert_file_has_content list-log "org.test.Hello"
+
+${FLATPAK} ${U} remote-modify --enable test-missing-gpg-repo
+${FLATPAK} ${U} remote-modify --enable test-wrong-gpg-repo
+${FLATPAK} ${U} remote-modify --enable test-gpg2-repo
+${FLATPAK} ${U} remote-modify --enable local-test-no-gpg-repo
+if [ x${USE_COLLECTIONS_IN_CLIENT-} != xyes ] ; then
+    ${FLATPAK} ${U} remote-modify --enable test-no-gpg-repo
+fi
+
+echo "ok missing remote name auto-corrects for install"
+
 ${FLATPAK} ${U} uninstall -y org.test.Platform org.test.Hello
 
 if ${FLATPAK} ${U} install -y test-missing-gpg-repo org.test.Platform 2> install-error-log; then
@@ -157,70 +196,6 @@ assert_file_has_content install-error-log "GPG signatures found, but none are in
 
 echo "ok fail with wrong gpg key"
 
-if [ x${USE_COLLECTIONS_IN_SERVER-} == xyes ] ; then
-    CID=org.test.Collection.test
-else
-    CID=""
-fi
-
-make_required_version_app () {
-    APP_ID=${1}
-    VERSION=${2}
-
-    REQUIRED_VERSION="${VERSION}" GPGARGS="${FL_GPGARGS}" $(dirname $0)/make-test-app.sh test ${APP_ID} "${CID}" > /dev/null
-}
-
-CURRENT_VERSION=`cat "$test_builddir/package_version.txt"`
-V=( ${CURRENT_VERSION//./ } )  # Split parts to array
-
-make_required_version_app org.test.SameVersion "${V[0]}.${V[1]}.${V[2]}"
-make_required_version_app org.test.NeedNewerMicro "${V[0]}.${V[1]}.$(expr ${V[2]} + 1)"
-make_required_version_app org.test.NeedNewerMinor "${V[0]}.$(expr ${V[1]} + 1).${V[2]}"
-make_required_version_app org.test.NeedNewerMajor "$(expr ${V[0]} + 1).${V[1]}.${V[2]}"
-make_required_version_app org.test.NeedOlderMinor "${V[0]}.$(expr ${V[1]} - 1).${V[2]}"
-make_required_version_app org.test.MultiVersionFallback "${V[0]}.${V[1]}.${V[2]};0.9.0;"
-make_required_version_app org.test.MultiVersionFallbackFail "${V[0]}.$(expr ${V[1]} + 1).${V[2]};0.9.0;"
-make_required_version_app org.test.MultiVersionOk "${V[0]}.$(expr ${V[1]} + 1).0;${V[0]}.${V[1]}.${V[2]};"
-make_required_version_app org.test.MultiVersionNotOk "${V[0]}.$(expr ${V[1]} + 1).0;${V[0]}.${V[1]}.$(expr ${V[2]} + 1);"
-
-update_repo test "${CID}"
-
-${FLATPAK} ${U} install -y test-repo org.test.SameVersion
-${FLATPAK} ${U} install -y test-repo org.test.NeedOlderMinor
-
-if ${FLATPAK} ${U} install -y test-repo org.test.NeedNewerMicro 2> install-error-log; then
-    assert_not_reached "Should not be able to install with wrong micro version"
-fi
-assert_file_has_content install-error-log "needs a later flatpak version"
-
-if ${FLATPAK} ${U} install -y test-repo org.test.NeedNewerMinor 2> install-error-log; then
-    assert_not_reached "Should not be able to install with wrong minor version"
-fi
-assert_file_has_content install-error-log "needs a later flatpak version"
-
-if ${FLATPAK} ${U} install -y test-repo org.test.NeedNewerMajor 2> install-error-log; then
-    assert_not_reached "Should not be able to install with wrong micro version"
-fi
-assert_file_has_content install-error-log "needs a later flatpak version"
-
-${FLATPAK} ${U} install -y test-repo org.test.MultiVersionFallback
-
-if ${FLATPAK} ${U} install -y test-repo org.test.MultiVersionFallbackFail 2> install-error-log; then
-    assert_not_reached "Should not be able to install with wrong fallback version"
-fi
-assert_file_has_content install-error-log "needs a later flatpak version"
-
-${FLATPAK} ${U} install -y test-repo org.test.MultiVersionOk
-
-if ${FLATPAK} ${U} install -y test-repo org.test.MultiVersionNotOk 2> install-error-log; then
-    assert_not_reached "Should not be able to install with wrong multi version"
-fi
-assert_file_has_content install-error-log "needs a later flatpak version"
-
-${FLATPAK} ${U} uninstall -y --all
-
-echo "ok handles version requirements"
-
 ${FLATPAK} ${U} remotes -d | grep ^test-repo > repo-info
 assert_not_file_has_content repo-info "new-title"
 UPDATE_REPO_ARGS=--title=new-title update_repo
@@ -272,16 +247,16 @@ ${FLATPAK} repo --branches repos/test > branches-log
 assert_file_has_content branches-log "^app/org.test.Hello/.*eol=Reason2"
 
 ${FLATPAK} ${U} remote-ls -d test-repo > remote-ls-log
-assert_file_has_content remote-ls-log "^app/org.test.Hello/.*eol=Reason2"
+assert_file_has_content remote-ls-log "app/org.test.Hello/.*eol=Reason2"
 
-${FLATPAK} ${U} update -y org.test.Hello 2> update-log
-assert_file_has_content update-log "app/org.test.Hello/.*Reason2"
+${FLATPAK} ${U} update -y org.test.Hello > update-log
+assert_file_has_content update-log "org.test.Hello.*Reason2"
 
 ${FLATPAK} ${U} info org.test.Hello > info-log
-assert_file_has_content info-log "end-of-life: Reason2"
+assert_file_has_content info-log "End-of-life: Reason2"
 
 ${FLATPAK} ${U} list -d > list-log
-assert_file_has_content list-log "^org.test.Hello/.*eol=Reason2"
+assert_file_has_content list-log "org.test.Hello/.*eol=Reason2"
 
 ${FLATPAK} ${U} uninstall -y org.test.Hello org.test.Platform
 
@@ -315,16 +290,16 @@ fi
 ${FLATPAK} uninstall -y org.test.Platform org.test.Hello
 
 ${FLATPAK} ${U} list -d > list-log
-assert_not_file_has_content list-log "^org.test.Hello"
-assert_not_file_has_content list-log "^org.test.Platform"
+assert_not_file_has_content list-log "org.test.Hello"
+assert_not_file_has_content list-log "org.test.Platform"
 
 echo "ok uninstall vs installations"
 
 ${FLATPAK} ${U} install -y test-repo org.test.Hello
 
 ${FLATPAK} ${U} list -d > list-log
-assert_file_has_content list-log "^org.test.Hello"
-assert_file_has_content list-log "^org.test.Platform"
+assert_file_has_content list-log "org.test.Hello"
+assert_file_has_content list-log "org.test.Platform"
 
 if ${FLATPAK} ${U} uninstall -y org.test.Platform; then
     assert_not_reached "Should not be able to uninstall ${U} when there is a dependency installed"
@@ -334,16 +309,31 @@ ${FLATPAK} ${U} uninstall -y org.test.Hello
 ${FLATPAK} ${U} uninstall -y org.test.Platform
 
 ${FLATPAK} ${U} list -d > list-log
-assert_not_file_has_content list-log "^org.test.Hello"
-assert_not_file_has_content list-log "^org.test.Platform"
+assert_not_file_has_content list-log "org.test.Hello"
+assert_not_file_has_content list-log "org.test.Platform"
 
 echo "ok uninstall dependencies"
 
+${FLATPAK} ${U} install -y test-repo org.test.Hello
+
+# Note: This typo is only auto-corrected without user interaction because we're using -y
+${FLATPAK} ${U} uninstall -y hello
+${FLATPAK} ${U} uninstall -y platform
+
+echo "ok typo correction works for uninstall"
+
+${FLATPAK} ${U} install -y test-repo org.test.Hello master
+
+${FLATPAK} ${U} uninstall -y org.test.Hello master
+${FLATPAK} ${U} uninstall -y org.test.Platform master
+
+echo "ok install and uninstall support 'NAME BRANCH' syntax"
+
 ${FLATPAK} ${U} install -y --no-deploy test-repo org.test.Hello
 
 ${FLATPAK} ${U} list -d > list-log
-assert_not_file_has_content list-log "^org.test.Hello"
-assert_not_file_has_content list-log "^org.test.Platform"
+assert_not_file_has_content list-log "org.test.Hello"
+assert_not_file_has_content list-log "org.test.Platform"
 
 # Disable the remote to make sure we don't do i/o
 port=$(cat httpd-port-main)
@@ -355,16 +345,40 @@ ${FLATPAK} ${U} install -y --no-pull test-repo org.test.Hello
 ${FLATPAK}  ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo
 
 ${FLATPAK} ${U} list -d > list-log
-assert_file_has_content list-log "^org.test.Hello"
-assert_file_has_content list-log "^org.test.Platform"
+assert_file_has_content list-log "org.test.Hello"
+assert_file_has_content list-log "org.test.Platform"
 
 echo "ok install with --no-deploy and then --no-pull"
 
+${FLATPAK} ${U} uninstall -y org.test.Hello org.test.Platform
+
+${FLATPAK} ${U} install -y --no-deploy test-repo hello
+
+${FLATPAK} ${U} list -d > list-log
+assert_not_file_has_content list-log "org.test.Hello"
+assert_not_file_has_content list-log "org.test.Platform"
+
+# Disable the remote to make sure we don't do i/o
+port=$(cat httpd-port-main)
+${FLATPAK}  ${U} remote-modify --url="http://127.0.0.1:${port}/disable-test" test-repo
+
+# Note: The partial ref is only auto-corrected without user interaction because we're using -y
+${FLATPAK} ${U} install -y --no-pull test-repo hello
+
+# re-enable remote
+${FLATPAK}  ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo
+
+${FLATPAK} ${U} list -d > list-log
+assert_file_has_content list-log "org.test.Hello"
+assert_file_has_content list-log "org.test.Platform"
+
+echo "ok install with --no-deploy and then --no-pull works with typo correction"
+
 ${FLATPAK} uninstall -y --all
 
 ${FLATPAK} ${U} list -d > list-log
-assert_not_file_has_content list-log "^org.test.Hello"
-assert_not_file_has_content list-log "^org.test.Platform"
+assert_not_file_has_content list-log "org.test.Hello"
+assert_not_file_has_content list-log "org.test.Platform"
 
 echo "ok uninstall --all"
 
